/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;

/**
 * Interface of the JMX Admin service.
 * 
 * @author Christophe DENEUX - Linagora
 */
public interface AdminServiceClient {

    /**
     * Name of the associated MBean
     */
    public static final String ADMIN_SERVICE_MBEAN_NAME = "Admin";

    /**
     * Get the version of the kernel of the PEtALS container
     * 
     * @see javax.jbi.management.AdminServiceMBean#getSystemInfo()
     * @return the version of PEtALS.
     * @throws AdminServiceErrorException
     */
    public String getSystemInfo() throws AdminServiceErrorException;

    /**
     * Get the Binding Components installed into the PEtALS Container
     * 
     * @see javax.jbi.management.AdminServiceMBean#getBindingComponents()
     * @return the binding components of the PEtALS Container
     * @throws AdminServiceErrorException
     */
    public ObjectName[] getBindingComponents() throws AdminServiceErrorException;

    /**
     * Get the Services Engines installed into the PEtALS Container
     * 
     * @return the services engines of the PEtALS Container
     * @throws AdminServiceErrorException
     */
    public ObjectName[] getEngineComponents() throws AdminServiceErrorException;

    /**
     * Get the Component with the given name
     * 
     * @return null if no component has been found, the {@link ObjectName} of
     *         the component otherwise
     * @throws AdminServiceErrorException
     *             if an error occurs while performing the action
     */
    public ObjectName getComponentByName(String component) throws AdminServiceErrorException;

    /**
     * Check if a given JBI component is a Binding Component.
     * 
     * @param componentName
     *            the unique name of the component; must be non-null and
     *            non-empty
     * @return <code>true</code> if the component is a binding component;
     *         <code>false</code> if the component is a service engine or if
     *         there is no component with the given <code>componentName</code>
     *         installed in the JBI system
     * 
     * @throws AdminServiceErrorException
     *             An error occurs while performing the action
     */
    public boolean isBinding(final String componentName) throws AdminServiceErrorException;

    /**
     * Check if a given JBI component is a Service Engine.
     * 
     * @param componentName
     *            the unique name of the component; must be non-null and
     *            non-empty
     * @return <code>true</code> if the component is a service engine;
     *         <code>false</code> if the component is a binding component or if
     *         there is no component with the given <code>componentName</code>
     *         installed in the JBI system
     * 
     * @throws AdminServiceErrorException
     *             An error occurs while performing the action
     */
    public boolean isEngine(final String componentName) throws AdminServiceErrorException;

    public boolean isExchangeWithConsumerOkayForComponent(final String componentName,
            final ServiceEndpoint providerEndpoint, final MessageExchange exchange)
            throws AdminServiceErrorException;

    public boolean isExchangeWithProviderOkayForComponent(final String componentName,
            final ServiceEndpoint providerEndpoint, final MessageExchange exchange)
            throws AdminServiceErrorException;
}
