/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import java.net.URL;

import javax.jbi.management.DeploymentServiceMBean;

import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.IllegalServiceAssemblyStateException;
import org.ow2.petals.jmx.api.api.exception.ServiceAssemblyDoesNotExistException;

/**
 * Interface of the JMX deployment service.
 * 
 * @author Christophe Deneux - Linagora
 */
public interface DeploymentServiceClient {

    /**
     * Service assembly states
     */
    // TODO: To share with the Petals kernel side
    public enum State {

        STARTED("Started"), STOPPED("Stopped"), SHUTDOWN("Shutdown"), UNKNOWN("Unknown");

        private final String label;

        private State(final String label) {
            this.label = label;
        }

        public final static State parse(final String label)
                throws IllegalServiceAssemblyStateException {
            for (final State state : State.values()) {
                if (state.label.equals(label)) {
                    return state;
                }
            }
            // No existing state matches the value
            throw new IllegalServiceAssemblyStateException(label);
        }

        @Override
        public final String toString() {
            return this.label;
        }

    }

    /**
     * Name of the associated MBean
     */
    public static final String DEPLOYMENT_SERVICE_MBEAN_NAME = "Deployment";

    /**
     * Deploy a service assembly.
     * 
     * @see DeploymentServiceMBean#deploy(String)
     * @param packageURL
     * @throws DeploymentServiceErrorException
     */
    public String deploy(final URL packageURL) throws DeploymentServiceErrorException;

    /**
     * Force the undeployment of the service assembly.
     * 
     * @see DeploymentServiceMBean#undeploy(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws DeploymentServiceErrorException
     */
    public boolean forceUndeploy(final String serviceAssemblyName)
            throws DeploymentServiceErrorException;

    /**
     * Get deployed service assemblies.
     * 
     * @return an array of String containing all the deployed SA. Non-null; may be empty.
     * @throws DeploymentServiceErrorException
     */
    public String[] getDeployedServiceAssemblies() throws DeploymentServiceErrorException;

    /**
     * Get service assemblies containing service units running on a component.
     * 
     * @return an array of String containing all the deployed SA for the given
     *         component.
     * @param componentName
     * @throws DeploymentServiceErrorException
     */
    public String[] getDeployedServiceAssembliesForComponent(final String componentName)
            throws DeploymentServiceErrorException;

    /**
     * Get the service assembly descriptor.
     * 
     * @return the descriptor in String form
     * @throws DeploymentServiceErrorException
     */
    public String getServiceAssemblyDescriptor(final String serviceAssemblyName)
            throws DeploymentServiceErrorException;

    /**
     * Get Service Unit embedded in the given Service Assembly.
     * 
     * @param serviceAssemblyName
     * @return an array of SU names
     * @throws DeploymentServiceErrorException
     */
    public String[] getServiceUnitForServiceAssembly(final String serviceAssemblyName)
            throws DeploymentServiceErrorException;

    /**
     * Get the current state of a JBI service assembly.
     * 
     * @param serviceAssemblyName
     *            Service assembly identifier.
     * @return the current state of the JBI service assembly.
     * @throws ServiceAssemblyDoesNotExistException
     *             The service assembly does not exist
     * @throws DeploymentServiceErrorException
     */
    public State getState(final String serviceAssemblyName)
            throws ServiceAssemblyDoesNotExistException, DeploymentServiceErrorException;

    /**
     * Check if a service assembly is deployed.
     * 
     * @param serviceAssembly
     *            The service assembly identifier
     * @return true if the service assembly; false if the service assembly is
     *         not deploy or if <code>serviceAssembly</code> is null.
     * @throws DeploymentServiceErrorException
     */
    public boolean isServiceAssemblyDeployed(final String serviceAssembly)
            throws DeploymentServiceErrorException;

    /**
     * Shutdown a service assembly.
     * 
     * @see DeploymentServiceMBean#shutDown(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws DeploymentServiceErrorException
     */
    public String shutdown(final String serviceAssemblyName) throws DeploymentServiceErrorException;

    /**
     * Shutdown all service assemblies.
     * 
     * @throws DeploymentServiceErrorException
     */
    public String[] shutdownAllServiceAssemblies() throws DeploymentServiceErrorException;

    /**
     * Start a service assembly.
     * 
     * @see DeploymentServiceMBean#start(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws DeploymentServiceErrorException
     */
    public String start(final String serviceAssemblyName) throws DeploymentServiceErrorException;

    /**
     * Start all service assemblies.
     * 
     * @throws DeploymentServiceErrorException
     */
    public String[] startAllServiceAssemblies() throws DeploymentServiceErrorException;

    /**
     * Stop a service assembly.
     * 
     * @see DeploymentServiceMBean#stop(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws DeploymentServiceErrorException
     */
    public String stop(final String serviceAssemblyName) throws DeploymentServiceErrorException;

    /**
     * Stop all service assemblies.
     * 
     * @throws DeploymentServiceErrorException
     */
    public String[] stopAllServiceAssemblies() throws DeploymentServiceErrorException;

    /**
     * Undeploy a service assembly.
     * 
     * @see DeploymentServiceMBean#undeploy(String)
     * @param serviceAssemblyName
     *            service assembly identifier
     * @throws DeploymentServiceErrorException
     */
    public String undeploy(final String serviceAssemblyName) throws DeploymentServiceErrorException;

    /**
     * Undeploy all service assemblies.
     * 
     * @throws DeploymentServiceErrorException
     */
    public String[] undeployAllServiceAssemblies(boolean force)
            throws DeploymentServiceErrorException;

}
