/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import java.net.URL;

import javax.jbi.management.InstallationServiceMBean;

import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;

/**
 * Interface of the JMX installation service.
 * 
 * @author Christophe Deneux - Linagora
 */
public interface InstallationServiceClient {

    /**
     * Name of the associated MBean
     */
    public static final String INSTALLATION_SERVICE_MBEAN_NAME = "Installation";

    /**
     * 
     * Force unload of installer. This method forces the installation of the
     * related component, and then forces the unload of its installer.
     * 
     * @see org.ow2.petals.clientserverapi.jbi.management.installation.InstallationService#forceUnloadInstaller(String)
     * @param componentName
     * @throws InstallationServiceErrorException
     */
    public boolean forceUnloadInstaller(final String componentName)
            throws InstallationServiceErrorException;

    /**
     * Get the components that use a shared library. This API is not defined by
     * JBI.
     * 
     * @return All component identifiers using the provided shared-library.
     * @throws InstallationServiceErrorException
     */
    public String[] getInstalledComponentsForSharedlibrary(final String sharedLibraryName,
            final String sharedLibraryVersion)
            throws InstallationServiceErrorException;

    /**
     * Get all installed shared libraries. This API is not defined by JBI.
     * 
     * @return All installed shared libraries. Each shared library returned is composed of two items: name and version.
     * @throws InstallationServiceErrorException
     */
    public String[][] getInstalledSharedLibraries() throws InstallationServiceErrorException;

    /**
     * Get all existing installers. This API is not defined by JBI.
     * 
     * @return All installed shared libraries.
     * @throws InstallationServiceErrorException
     */
    public String[] getInstallers() throws InstallationServiceErrorException;

    /**
     * Install a shared-library.
     * 
     * @see InstallationServiceMBean#installSharedLibrary(String)
     * @return The unique name of the shared library installed.
     * @throws InstallationServiceErrorException
     */
    public String installSharedLibrary(final URL sharedLibraryURL)
            throws InstallationServiceErrorException;

    /**
     * Load an installer.
     * 
     * @see InstallationServiceMBean#loadInstaller(String)
     * @param componentName
     * @return The component installer if it exists. Cannot be <code>null</code>.
     * @throws InstallerComponentDoesNotExistException
     *             No component installer identified by
     *             <code>componentName</code> was loaded.
     * @throws InstallationServiceErrorException
     */
    public InstallerComponentClient loadInstaller(final String componentName)
            throws InstallerComponentDoesNotExistException, InstallationServiceErrorException;

    /**
     * Load a new installer.
     * 
     * @see InstallationServiceMBean#loadNewInstaller(String)
     * @param packageURL
     * @throws InstallationServiceErrorException
     */
    public InstallerComponentClient loadNewInstaller(final URL packageURL)
            throws InstallationServiceErrorException;

    /**
     * Shutdown all the component.
     * 
     * @return the list of component names that has been shut down
     * @throws InstallationServiceErrorException
     */
    public String[] shutdownAllComponents() throws InstallationServiceErrorException;

    /**
     * Shutdown the component.
     * 
     * @param componentName
     * @return {@code true} if the shut down was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean shutdownComponent(String componentName) throws InstallationServiceErrorException;

    /**
     * Start all the component.
     * 
     * @return the list of component names that has been started
     * @throws InstallationServiceErrorException
     */
    public String[] startAllComponents() throws InstallationServiceErrorException;

    /**
     * Start the component.
     * 
     * @param componentName
     * @return {@code true} if the start was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean startComponent(String componentName) throws InstallationServiceErrorException;

    /**
     * Stop all the component.
     * 
     * @return the list of component names that has been stopped
     * @throws InstallationServiceErrorException
     */
    public String[] stopAllComponents() throws InstallationServiceErrorException;

    /**
     * Stop the component.
     * 
     * @param componentName
     * @return {@code true} if the stop was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean stopComponent(String componentName) throws InstallationServiceErrorException;

    /**
     * Uninstall all the component.
     * 
     * @return the list of component names that has been uninstalled
     * @throws InstallationServiceErrorException
     */
    public String[] uninstallAllComponents() throws InstallationServiceErrorException;

    /**
     * Uninstall all shared-library.
     * 
     * @return the list of the Shared Libraries names that has been uninstalled
     * @throws InstallationServiceErrorException
     */
    public String[] uninstallAllSharedLibrary() throws InstallationServiceErrorException;

    /**
     * Uninstall a shared Library identified by its name and version.
     * 
     * @return true if the uninstall was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean uninstallSharedLibrary(final String sharedLibraryName, final String sharedLibraryVersion)
            throws InstallationServiceErrorException;

    /**
     * Unload all the component installers.
     * 
     * @return the list of component names whose installers has been unloaded.
     * @throws InstallationServiceErrorException
     */
    public String[] unloadAllInstallers(boolean force) throws InstallationServiceErrorException;

    /**
     * 
     * Unload a component installer. The component is deleted as well.
     * 
     * @see InstallationServiceMBean#unloadInstaller(String, boolean)
     * @param componentName
     * @throws InstallationServiceErrorException
     */
    public boolean unloadInstaller(final String componentName)
            throws InstallationServiceErrorException;
}
