/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import javax.jbi.management.InstallerMBean;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.configuration.component.InstallationConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.configuration.component.exception.ConfigurationComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentErrorException;

/**
 * Interface of a JMX component associated to a JBI component installer.
 * 
 * @author Christophe Deneux - Capgemini Sud
 */
public interface InstallerComponentClient {

    /**
     * Install the component.
     * 
     * @see InstallerMBean#install()
     * @throws InstallerComponentErrorException
     */
    public void install() throws InstallerComponentErrorException;

    /**
     * Uninstall the component.
     * 
     * @see InstallerMBean#uninstall()
     * @throws InstallerComponentErrorException
     */
    public void uninstall() throws InstallerComponentErrorException;

    /**
     * Get the current installation state of the JBI component
     * 
     * @see InstallerMBean#isInstalled()
     * @return the current state of the JBI component
     * @throws InstallerComponentErrorException
     */
    public boolean isInstalled() throws InstallerComponentErrorException;

    /**
     * Get the root path to the component installation.
     * 
     * @see InstallerMBean#getInstallRoot()
     * @return the root path
     * @throws InstallerComponentErrorException
     */
    public String getInstallRoot() throws InstallerComponentErrorException;

    /**
     * Get the MBean name of this installer.
     * 
     * @return
     */
    public ObjectName getMBeanName();
    
    /**
     * Retrieve the configuration component client associated to a JBI component installer.
     * 
     * This implements generic access to the configuration service (set/get JMX attributes).
     * 
     * For interface-specific configuration (JMX operations), see {@link #getInstallationConfigurationClient(Class)}.
     * 
     * @return The installation configuration MBean or <code>null</code> if not available
     * @throws ConfigurationComponentDoesNotExistException
     *             No installation configuration component MBean exists.
     * @throws ConfigurationComponentErrorException
     *             An error occurs working with the installation configuration component MBean
     * @throws InstallerComponentErrorException
     *             An error occurs retrieving the installation configuration component MBean
     */
    public InstallationConfigurationComponentClient getInstallationConfigurationClient()
            throws ConfigurationComponentDoesNotExistException, ConfigurationComponentErrorException,
            InstallerComponentErrorException;

    /**
     * Retrieve the configuration component client associated to a JBI component installer.
     * 
     * This implements interface-specific configuration (JMX operations).
     * 
     * For generic access to the configuration service (set/get JMX attributes), see
     * {@link #getInstallationConfigurationClient()}
     * 
     * @param clazz
     *            the {@link Class} implemented by the component for configuration
     * @return The installation configuration MBean or <code>null</code> if not available
     * @throws ConfigurationComponentDoesNotExistException
     *             No installation configuration component MBean exists.
     * @throws InstallerComponentErrorException
     *             An error occurs retrieving the installation configuration component MBean
     */
    public <T> T getInstallationConfigurationClient(Class<T> clazz) throws ConfigurationComponentDoesNotExistException,
            InstallerComponentErrorException;
}
