/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import java.net.InetAddress;

import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointDirectoryServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.SeFlowableMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.LocalTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectFilter;
import org.ow2.petals.jmx.api.api.monitoring.defect.DefectListener;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.defect.exception.NoDefectListenerException;

/**
 * Interface of a Petals JMX Client.
 * 
 * @author Christophe Deneux - Linagora
 */
public interface JMXClient {

    public final static int DEFAULT_PORT = 7700;

    /**
     * The name of the RMI Stub in the RMI Registry used by JMX
     */
    public final static String PETALS_JMX_RMI_STUB_NAME = "PetalsJMX";

    /**
     * @return The IP address of the JMX Server on which the JMX client is connected.
     * @throws NotRemoteJMXClientException
     *             The JMX connection is not a remote connection.
     */
    public InetAddress getHost() throws NotRemoteJMXClientException;

    /**
     * @return The TCP port of the JMX Server on which the JMX client is connected.
     * @throws NotRemoteJMXClientException
     *             The JMX connection is not a remote connection.
     */
    public int getPort() throws NotRemoteJMXClientException;

    /**
     * @return The username with which the JMX client is connected.
     * @throws NotRemoteJMXClientException
     *             The JMX connection is not a remote connection.
     */
    public String getUsername() throws NotRemoteJMXClientException;

    /**
     * @return The password of the username with which the JMX client is connected.
     * @throws NotRemoteJMXClientException
     *             The JMX connection is not a remote connection.
     */
    public String getPassword() throws NotRemoteJMXClientException;

    /**
     * Retrieve the Petals Admin service client.
     * 
     * @return The Petals Admin service client
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public PetalsAdminServiceClient getPetalsAdminServiceClient()
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException, ConnectionErrorException;

    /**
     * Retrieve the Admin service client.
     * 
     * @return The Admin service client
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public AdminServiceClient getAdminServiceClient()
            throws AdminDoesNotExistException, AdminServiceErrorException, ConnectionErrorException;

    /**
     * Retrieve the installation service client.
     * 
     * @return the installation service client
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     * @throws ConnectionErrorException
     */
    public InstallationServiceClient getInstallationServiceClient() throws InstallationServiceDoesNotExistException,
            InstallationServiceErrorException, ConnectionErrorException;

    /**
     * Retrieve the deployment service client.
     * 
     * @return the deployment service client
     * @throws DeploymentServiceDoesNotExistException
     * @throws DeploymentServiceErrorException
     * @throws ConnectionErrorException
     */
    public DeploymentServiceClient getDeploymentServiceClient()
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException, ConnectionErrorException;

    /**
     * Retrieve the component client associated to a JBI component.
     * 
     * @param componentName
     *            Component name
     * @return the component client
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public ComponentClient getComponentClient(final String componentName)
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException;

    /**
     * Retrieve the component client associated to a JBI component.
     * 
     * @param componentMBeanName
     *            MBean name of the component
     * @return the component client
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public ComponentClient getComponentClient(ObjectName componentMBeanName)
            throws ComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException;

    /**
     * Retrieve the component installer client associated to a JBI component.
     * 
     * @param componentName
     *            Component name
     * @return the installer component client
     * @throws InstallerComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public InstallerComponentClient getInstallerComponentClient(String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException, ConnectionErrorException;

    /**
     * Retrieve the endpoint directory client
     * 
     * @return the endpoint directory client
     * @throws ConnectionErrorException
     * @throws EndpointDirectoryServiceErrorException
     * @throws EndpointDirectoryServiceDoesNotExistException
     */
    public EndpointDirectoryClient getEndpointDirectoryClient() throws ConnectionErrorException,
            EndpointDirectoryServiceDoesNotExistException, EndpointDirectoryServiceErrorException;

    /**
     * Retrieve the system monitoring service client
     * 
     * @return the SystemMonitoring component client
     * @throws ConnectionErrorException
     * @throws SystemMonitoringServiceErrorException
     * @throws SystemMonitoringDoesNotExistException
     */
    public SystemMonitoringServiceClient getSystemMonitoringServiceClient() throws ConnectionErrorException,
            SystemMonitoringDoesNotExistException, SystemMonitoringServiceErrorException;

    /**
     * Retrieve the Logger service client
     * 
     * @return The Logger service client
     * @throws LoggerServiceErrorException
     * @throws LoggerServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    public LoggerServiceClient getLoggerServiceClient()
            throws LoggerServiceErrorException, LoggerServiceDoesNotExistException, ConnectionErrorException;

    /**
     * Retrieve the local transporter monitoring service client
     * 
     * @return The local transporter monitoring service client
     * @throws LocalTransporterMonitoringServiceErrorException
     * @throws LocalTransporterMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    public LocalTransporterMonitoringServiceClient getLocalTransporterMonitoringServiceClient()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException;

    /**
     * Retrieve the TCP transporter monitoring service client
     * 
     * @return The TCP transporter monitoring service client
     * @throws TcpTransporterMonitoringServiceErrorException
     * @throws TcpTransporterMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    public TcpTransporterMonitoringServiceClient getTcpTransporterMonitoringServiceClient()
            throws TcpTransporterMonitoringServiceErrorException, TcpTransporterMonitoringServiceDoesNotExistException,
            ConnectionErrorException;

    /**
     * Retrieve the monitoring service client of basic component based on the CDK and that does not provide other
     * monitoring metrics
     * 
     * @param componentIdentifier
     *            The unique identifier of the component
     * 
     * @return The monitoring service client associated to a basic component
     * @throws ComponentMonitoringServiceErrorException
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    // TODO: When we will be able to identify correctly a remote component monitoring MBean, this method could be remove
    // and the ComponentMonitoringServiceClient could be
    // retrieved through a new method JMXClient.getComponentClient(String).getMonitoringServiceClient() and
    // inheritance.
    public ComponentMonitoringServiceClient getComponentMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException;

    /**
     * Retrieve the Petals BC SOAP monitoring service client
     * 
     * @param componentIdentifier
     *            The unique identifier of the Petals BC SOAP
     * 
     * @return The Petals BC SOAP monitoring service client
     * @throws ComponentMonitoringServiceErrorException
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    // TODO: When we will be able to identify correctly a remote component monitoring MBean as a monitoring MBean of
    // Petals BC Soap, this method could be remove and the ComponentMonitoringServiceClient could be retrieved through a
    // new method JMXClient.getComponentClient(String).getMonitoringServiceClient() and inheritance.
    // TODO: Rename the method as 'getBcSoapMonitoringServiceClient'
    // TODO: Rename the returned class as 'BcSoapMonitoringServiceClient'
    public PetalsBcSoapMonitoringServiceClient getPetalsBcSoapMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException;

    /**
     * Retrieve the Petals SE Flowable monitoring service client
     * 
     * @param componentIdentifier
     *            The unique identifier of the Petals SE Flowable
     * 
     * @return The Petals SE Flowable monitoring service client
     * @throws ComponentMonitoringServiceErrorException
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    // TODO: When we will be able to identify correctly a remote component monitoring MBean as a monitoring MBean of
    // Petals SE Flowable, this method could be remove and the ComponentMonitoringServiceClient could be retrieved
    // through a
    // new method JMXClient.getComponentClient(String).getMonitoringServiceClient() and inheritance.
    public SeFlowableMonitoringServiceClient getSeFlowableMonitoringServiceClient(final String componentIdentifier)
            throws ComponentMonitoringServiceErrorException, ComponentMonitoringServiceDoesNotExistException,
            ConnectionErrorException;

    /**
     * Subscribe to defects of a Petals component that was detected by monitoring sensors
     * 
     * @param componentIdentifier
     *            Unique identifier of the Petals component
     * @param defectListener
     *            Defect listener
     * @param defectFilter
     *            Defect filter. No filter applied if <code>null</code>.
     * @throws DefectListenerException
     *             An error occurs subscribing to the defects of the BC SOAP
     * @throws DefectEmitterDoesNotExistsException
     *             The BC SOAP does not exist
     */
    public void subscribeToComponentMonitoringService(final String componentIdentifier,
            final DefectListener defectListener, final DefectFilter defectFilter)
                    throws DefectListenerException, DefectEmitterDoesNotExistsException;

    /**
     * Unsubscribes to defects of a Petals component that was detected by monitoring sensors
     * 
     * @param componentIdentifier
     *            Unique identifier of the Petals component
     * @throws NoDefectListenerException
     * @throws DefectListenerException
     * @throws DefectEmitterDoesNotExistsException
     */
    public void unsubscribeToComponentMonitoringService(final String componentIdentifier)
            throws NoDefectListenerException, DefectEmitterDoesNotExistsException, DefectListenerException;

    /**
     * Retrieve the Topology service client
     * 
     * @return The Topology service client
     * @throws TopologyServiceErrorException
     * @throws TopologyServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    public TopologyServiceClient getTopologyServiceClient()
            throws TopologyServiceErrorException, TopologyServiceDoesNotExistException, ConnectionErrorException;

    /**
     * Release the connection to the JMX server
     * 
     * @throws ConnectionErrorException
     */
    public void disconnect() throws ConnectionErrorException;
}
