/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;

import javax.management.MBeanServer;

import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * The Petals JMX API Factory
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class PetalsJmxApiFactory {

    /**
     * Name of the property used to select the implementation of Petals JMX API.
     */
    public final static String PROPERTY_NAME = PetalsJmxApiFactory.class.getName();

    /**
     * Lock to initialize {@link #API_INSTANCE}
     */
    private final static Object INITIALIZATION_LOCK = new Object();

    private static PetalsJmxApiFactory API_INSTANCE = null;

    /**
     * <p>
     * Get the Petals JMX API instance using a implementation according to the
     * following rule:
     * </p>
     * <ul>
     * <li>first, if a property with the service name is set, it is used to look
     * for the service implementation,</li>
     * <li>otherwise the service implementation is looking for in the current
     * classloader.</li>
     * </ul>
     * <p>
     * On the first call, the Petals JMX API instance is created.
     * </p>
     * 
     * @throws DuplicatedServiceException
     *             The service to look for is duplicated.
     * @throws MissingServiceException
     *             Unable to find the service.
     */
    public static final PetalsJmxApiFactory getInstance() throws DuplicatedServiceException,
            MissingServiceException {
        synchronized (INITIALIZATION_LOCK) {
            if (API_INSTANCE == null) {
                final String overridenImplementation = System
                        .getProperty(PetalsJmxApiFactory.PROPERTY_NAME);
                if (overridenImplementation == null || overridenImplementation.trim().isEmpty()) {
                    final ServiceLoader<PetalsJmxApiFactory> serviceLoader = ServiceLoader
                            .load(PetalsJmxApiFactory.class);
                    final Iterator<PetalsJmxApiFactory> servicesIterator = serviceLoader.iterator();
                    try {
                        if (servicesIterator.hasNext()) {
                            final PetalsJmxApiFactory serviceLoaded = servicesIterator.next();
                            if (servicesIterator.hasNext()) {
                                throw new DuplicatedServiceException(PetalsJmxApiFactory.class);
                            } else {
                                API_INSTANCE = serviceLoaded;
                            }
                        } else {
                            throw new MissingServiceException(PetalsJmxApiFactory.class);
                        }
                    } catch (final ServiceConfigurationError e) {
                        throw new MissingServiceException(PetalsJmxApiFactory.class, e);
                    }
                } else {
                    try {
                        API_INSTANCE = ReflectionHelper.newInstance(overridenImplementation, null);
                    } catch (final InvocationTargetException e) {
                        throw new MissingServiceException(PetalsJmxApiFactory.class, e);
                    }
                }
            }

            return API_INSTANCE;
        }
    }

    /**
     * Free resources allocated by the Petals JMX Client API instance
     */
    public static final void close() {
        synchronized (INITIALIZATION_LOCK) {
            API_INSTANCE = null;
        }
    }

    /**
     * Create a new JMX client connected to the remote petals ESB.
     * 
     * @param host
     *            The hostname where the remote Petals ESB container runs. If set to <code>null</code>, the host name is
     *            <code>localhost</code>.
     * @param port
     *            The JMX port of the remote Petals ESB container. If set to <code>null</code>, the default port
     *            {@link JMXClient#DEFAULT_PORT} is used.
     * @param username
     *            Username of the credentials required to establish a connection to the remote Petals ESB container. A
     *            <code>null</code> value used with a JMX server protected by credentials will raise a security error,
     *            but is accepted with a JMX server not secured.
     * @param password
     *            Password of the credentials required to establish a connection to the remote Petals ESB container. If
     *            set to <code>null</code>, an empty value is used.
     * 
     * @return a JMX client connected to the remote Petals ESB
     * 
     * @throws ConnectionErrorException
     *             if there is no current client connection to the remote petals ESB
     */
    public abstract JMXClient createJMXClient(final String host, final Integer port,
            final String username, final String password) throws ConnectionErrorException;

    /**
     * Creates a new JMX client connected to an embedded JMX server.
     * 
     * @param mbeanServer
     *            The embedded JMX server
     * @throws ConnectionErrorException
     *             An error occurs establishing the JMX connection
     */
    public abstract JMXClient createJMXClient(final MBeanServer mbeanServer)
            throws ConnectionErrorException;

}
