/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.jmx.api.api.monitoring.component.framework;

import java.util.Map;

import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;

/**
 * Interface of the JMX service providing monitoring metrics associated to the
 * CDK level.
 * 
 * @author Christophe Deneux - Linagora
 */
public interface ComponentMonitoringServiceClient {

    /**
     * Get the metric 'service provider invocations'
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<ServiceProviderInvocationKey, Long> getServiceProviderInvocationsCount()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Get the metric 'relative response times of service provider invocations'.
     * </p>
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<ServiceProviderInvocationKey, Long[]> getServiceProviderInvocationsResponseTimes()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Returns the relative response time values of service provider invocations on a time sample group by interface,
     * service, operation and execution status.
     * </p>
     * <p>
     * The key part 'message exchange pattern' is null because of grouping on interface, service, operation and
     * execution status.
     * </p>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>1st: the max response time on the time sample,</li>
     * <li>2nd: the average,</li>
     * <li>3rd: the min value,</li>
     * <li>4th the 10-percentile,</li>
     * <li>5th the 50-percentile,</li>
     * <li>6th: the 90-percentile.</li>
     * </ul>
     * 
     * @return The response time values of service provider invocations. All execution status are contained in the map,
     *         even if set to 0.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<ServiceProviderInvocationKey, Long[]> getServiceProviderInvocationsResponseTimesGroupedByItfSvcOpAndExecStatus()
            throws ComponentMonitoringServiceErrorException;

    /**
     * @return The max number of active thread of the message exchange acceptor pool.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Integer getMexAcceptorThreadPoolActiveThreadsMaxSize()
            throws ComponentMonitoringServiceErrorException;

    /**
     * @return The current number of active thread of the message exchange acceptor pool.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Integer getMexAcceptorThreadPoolActiveThreadsCurrentSize()
            throws ComponentMonitoringServiceErrorException;

    /**
     * @return The current number of working message exchange acceptors.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Long getMexAcceptorWorkingCurrent() throws ComponentMonitoringServiceErrorException;

    /**
     * @return The max number of working message exchange acceptors.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Long getMexAcceptorWorkingMax() throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Returns the relative durations, in nanoseconds, of working message exchange acceptors on a time sample.
     * </p>
     * <p>
     * Each value of the returned array is, in milliseconds:
     * </p>
     * <ul>
     * <li>1st: the max response time on the time sample,</li>
     * <li>2nd: the average,</li>
     * <li>3rd: the min value,</li>
     * <li>4th the 10-percentile,</li>
     * <li>5th the 50-percentile,</li>
     * <li>6th: the 90-percentile.</li>
     * </ul>
     * 
     * @return Relative durations of working message exchange acceptors.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Long[] getMexAcceptorRelativeDurations() throws ComponentMonitoringServiceErrorException;

    /**
     * @return Get the current metrics of the message exchange processor thread
     *         pool.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public ThreadPoolMetrics getMexProcessorThreadPoolMetrics()
            throws ComponentMonitoringServiceErrorException;

}
