/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api.monitoring.component.soap;

import java.util.Map;

import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;

/**
 * Interface of the JMX service providing Petals BC Soap monitoring metrics.
 * 
 * @author Christophe Deneux - Linagora
 */
public interface PetalsBcSoapMonitoringServiceClient extends ComponentMonitoringServiceClient {

    /**
     * Get the metric 'incoming web-service requests'
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<IncomingWsRequestKey, Long> getIncomingWsRequestsCount()
            throws ComponentMonitoringServiceErrorException;

    /**
     * Get the metric 'incoming "information" HTTP-requests'
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Long getInformationHttpRequestsCount()
            throws ComponentMonitoringServiceErrorException;

    /**
     * Get the metric 'incoming "unknown" HTTP-requests'
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Long getUnknownHttpRequestsCount() throws ComponentMonitoringServiceErrorException;

    /**
     * Get the metric 'incoming "service contracts" HTTP-requests'
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<String, Long> getServiceContractsHttpRequestsCount() throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Get the metric 'relative response times of incoming web-service requests'.
     * </p>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>1st: the max response time on the time sample,</li>
     * <li>2nd: the average,</li>
     * <li>3rd: the min value,</li>
     * <li>4th the 10-percentile,</li>
     * <li>5th the 50-percentile,</li>
     * <li>6th: the 90-percentile.</li>
     * </ul>
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<IncomingWsRequestKey, Long[]> getIncomingWsRequestsResponseTimes()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Returns the relative response time values of incoming ws-requests on a
     * time sample group by ws-path, ws-operation and execution status.
     * </p>
     * <p>
     * The key part 'client IP address' is null because of grouping on ws-path,
     * ws-operation and execution status
     * </p>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>1st: the max response time on the time sample,</li>
     * <li>2nd: the average,</li>
     * <li>3rd: the min value,</li>
     * <li>4th the 10-percentile,</li>
     * <li>5th the 50-percentile,</li>
     * <li>6th: the 90-percentile.</li>
     * </ul>
     * 
     * @return The response time values of incoming ws-requests. All execution
     *         status are contained in the map, even if set to 0.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<IncomingWsRequestKey, Long[]> getIncomingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException;



    /**
     * Get the current metrics of the HTTP server thread pool
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public ThreadPoolMetrics getHttpServerThreadPoolMetrics()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Get the max number of client in use of the ws-client pools, per external
     * web-service, operation and message exchange pattern.
     * </p>
     * 
     * @return The max number of client in use.
     * 
     * @throws ComponentMonitoringServiceErrorException
     * @see WsClientPoolKey to access values composing the map key.
     */
    public Map<WsClientPoolKey, Long> getWsClientPoolClientsInUseMax()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Get the number of client currently in use of the ws-client pools, per
     * external web-service, operation and message exchange pattern.
     * </p>
     * 
     * @return The number of client currently in use.
     * 
     * @throws ComponentMonitoringServiceErrorException
     * @see WsClientPoolKey to access values composing the map key.
     */
    public Map<WsClientPoolKey, Long> getWsClientPoolClientsInUseCurrent()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Get the number of exhaustions of the ws-client pools, per external
     * web-service, operation and message exchange pattern.
     * </p>
     * 
     * @return The number of exhaustions
     * 
     * @throws ComponentMonitoringServiceErrorException
     * @see WsClientPoolKey to access values composing the map key.
     */
    public Map<WsClientPoolKey, Long> getWsClientPoolExhaustions()
            throws ComponentMonitoringServiceErrorException;

    /**
     * Get the metric 'outgoing web-service requests'
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<OutgoingWsRequestKey, Long> getOutgoingWsRequestsCount()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Get the metric 'relative response time of outgoing web-service requests'.
     * </p>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>1st: the max response time on the time sample,</li>
     * <li>2nd: the average,</li>
     * <li>3rd: the min value,</li>
     * <li>4th the 10-percentile,</li>
     * <li>5th the 50-percentile,</li>
     * <li>6th: the 90-percentile.</li>
     * </ul>
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<OutgoingWsRequestKey, Long[]> getOutgoingWsRequestsResponseTimes()
            throws ComponentMonitoringServiceErrorException;

    /**
     * <p>
     * Returns the relative response time values of outgoing requests on a time
     * sample group by external web-service URL, ws-operation and execution
     * status.
     * </p>
     * <p>
     * The key part 'mep' is null because of grouping on external web-service
     * URL, ws-operation and execution status
     * </p>
     * <p>
     * Each value of the map entry is an array of response times where:
     * </p>
     * <ul>
     * <li>1st: the max response time on the time sample,</li>
     * <li>2nd: the average,</li>
     * <li>3rd: the min value,</li>
     * <li>4th the 10-percentile,</li>
     * <li>5th the 50-percentile,</li>
     * <li>6th: the 90-percentile.</li>
     * </ul>
     * 
     * @return The relative response time values of outgoing ws-requests. All
     *         execution status are contained in the map, even if set to 0.
     * 
     * @throws ComponentMonitoringServiceErrorException
     */
    public Map<OutgoingWsRequestKey, Long[]> getOutgoingWsRequestsResponseTimesGroupedBySvcAndExecStatus()
            throws ComponentMonitoringServiceErrorException;
}
