/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote;

import java.util.Map;

import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;

/**
 * Interface of the JMX service providing TCP transporter monitoring metrics.
 * 
 * @author Christophe Deneux - Linagora
 */
public interface TcpTransporterMonitoringServiceClient {

    /**
     * Name of the associated MBean
     */
    public static final String TCP_TRANSPORTER_MONITORING_MBEAN_NAME = "TcpTransporterMonitoring";

    // -----------------------------
    // Messages counters
    // -----------------------------

    /**
     * <p>
     * Get metrics about the number of messages sent to remote containers by the current container, per remote
     * container, since the last start of the transporter.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * </p>
     * <ul>
     * <li>the remote container name as defined at topology level,</li>
     * <li>the delivery status: PENDING, SUCCEEDED or ERROR.</li>
     * </ul>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getDeliveredOutgoingMessagesCount()
            throws TcpTransporterMonitoringServiceErrorException;

    /**
     * <p>
     * Get metrics about the number of messages received from remote containers by the current container, per remote
     * container, since the last start of the transporter.
     * </p>
     * <p>
     * The map key is composed of following ordered fields:
     * </p>
     * <ul>
     * <li>the remote container name as IP address,</li>
     * <li>the delivery status: PENDING, SUCCEEDED or ERROR.</li>
     * </ul>
     * 
     * @return The metrics.
     */
    public Map<String[], Long> getDeliveredIncomingMessagesCount() throws TcpTransporterMonitoringServiceErrorException;

    // -----------------------------
    // Connection counters
    // -----------------------------

    /**
     * 
     * <p>
     * Get the current metrics of the outgoing connection pools, per remote container. An outgoing connection is a
     * connection established from the current container to a remote one.
     * </p>
     * <p>
     * The map key is the remote container name.
     * </p>
     * 
     * @return Get the current metrics of the outgoing connection pools.
     * 
     * @throws TcpTransporterMonitoringServiceErrorException
     */
    public Map<String, OutgoingConnectionObjectPoolMetrics> getOutgoingConnectionPoolMetrics()
            throws TcpTransporterMonitoringServiceErrorException;

    /**
     * 
     * <p>
     * Get the current metrics of the incoming connections, per remote container. An incoming connection is a connection
     * established from a remote container to the current one.
     * </p>
     * <p>
     * The map key is the remote container name.
     * </p>
     * 
     * @return Get the current metrics of the incoming connections.
     * 
     * @throws TcpTransporterMonitoringServiceErrorException
     */
    public Map<String, IncomingConnectionObjectPoolMetrics> getIncomingConnectionPoolMetrics()
            throws TcpTransporterMonitoringServiceErrorException;
}
