/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;

import javax.management.MBeanServer;

import org.junit.Test;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.jmx.api.utest.PetalsJmxApiFactoryUTest;

/**
 * Unit tests of the Petals JMX client factory
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsJmxApiFactoryTest {

    /**
     * Dummy implementation of {@link PetalsJmxApiFactory} for duplication test.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public class PetalsJmxApiFactoryDummy extends PetalsJmxApiFactory {

        @Override
        public JMXClient createJMXClient(String host, Integer port, String username,
                String password) throws ConnectionErrorException {
            return null;
        }

        @Override
        public JMXClient createJMXClient(MBeanServer mbeanServer) throws ConnectionErrorException {
            return null;
        }
    }

    /**
     * <p>
     * Check the loading of the Petals JMX Client API implementation when only
     * one implementation is registered through the classpath.
     * </p>
     * <p>
     * Expected result: The implementation available on the classpath is laoded.
     * </p>
     */
    @Test
    public void testGetInstance_00() throws DuplicatedServiceException,
            MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsJmxApiFactory.class.getName())) {
                    return super.getResources("services/utest/"
                            + PetalsJmxApiFactory.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final PetalsJmxApiFactory petalsJmxApiFactory = PetalsJmxApiFactory.getInstance();
            assertTrue(petalsJmxApiFactory instanceof PetalsJmxApiFactoryUTest);
        } finally {
            PetalsJmxApiFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals JMX Client API implementation when no
     * implementation is registered through the classpath and the property is
     * not set.
     * </p>
     * <p>
     * Expected result: The exception {@link MissingServiceException} is thrown.
     * </p>
     */
    @Test(expected = MissingServiceException.class)
    public void testGetInstance_01() throws DuplicatedServiceException,
            MissingServiceException {

        try {
            PetalsJmxApiFactory.getInstance();
            fail("Exception " + MissingServiceException.class.getName() + " is not thrown !");
        } finally {
            PetalsJmxApiFactory.close();
        }
    }

    /**
     * <p>
     * Check the loading of the Petals JMX Client API implementation when no
     * implementation is registered through the classpath and the property is
     * set.
     * </p>
     * <p>
     * Expected result: The implementation defined by the property is loaded.
     * </p>
     */
    @Test
    public void testGetInstance_02() throws DuplicatedServiceException,
            MissingServiceException {

        System.setProperty(PetalsJmxApiFactory.PROPERTY_NAME,
                PetalsJmxApiFactoryUTest.class.getName());
        try {
            final PetalsJmxApiFactory petalsJmxApiFactory = PetalsJmxApiFactory.getInstance();
            assertTrue(petalsJmxApiFactory instanceof PetalsJmxApiFactoryUTest);
        } finally {
            PetalsJmxApiFactory.close();
            System.clearProperty(PetalsJmxApiFactory.PROPERTY_NAME);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals JMX Client API implementation when
     * several implementations are registered through the classpath.
     * </p>
     * <p>
     * Expected result: The exception {@link DuplicatedServiceException} is
     * thrown.
     * </p>
     */
    @Test(expected = DuplicatedServiceException.class)
    public void testGetInstance_03() throws DuplicatedServiceException,
            MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsJmxApiFactory.class.getName())) {
                    return super.getResources("services/duplicated/"
                            + PetalsJmxApiFactory.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            PetalsJmxApiFactory.getInstance();
            fail("Exception " + DuplicatedServiceException.class.getName() + " is not thrown !");
        } finally {
            PetalsJmxApiFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals JMX Client API implementation when
     * several implementations are registered through the classpath, and the
     * property is set to one of the implementation registered through the
     * classpath.
     * </p>
     * <p>
     * Expected result: The implementation defined by the property is loaded.
     * </p>
     */
    @Test
    public void testGetInstance_04() throws DuplicatedServiceException, MissingServiceException {

        System.setProperty(PetalsJmxApiFactory.PROPERTY_NAME,
                PetalsJmxApiFactoryUTest.class.getName());

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsJmxApiFactory.class.getName())) {
                    return super.getResources("services/duplicated/"
                            + PetalsJmxApiFactory.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final PetalsJmxApiFactory petalsJmxApiFactory = PetalsJmxApiFactory.getInstance();
            assertTrue(petalsJmxApiFactory instanceof PetalsJmxApiFactoryUTest);
        } finally {
            PetalsJmxApiFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
            System.clearProperty(PetalsJmxApiFactory.PROPERTY_NAME);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals Client JMX API implementation when only
     * one implementation is registered through the classpath, but the class
     * does not exist.
     * </p>
     * <p>
     * Expected result: The exception {@link MissingServiceException} is thrown.
     * </p>
     */
    @Test(expected = MissingServiceException.class)
    public void testGetInstance_05() throws DuplicatedServiceException, MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsJmxApiFactory.class.getName())) {
                    return super.getResources("services/class-doesnot-exist/"
                            + PetalsJmxApiFactory.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            PetalsJmxApiFactory.getInstance();
            fail("Exception " + MissingServiceException.class.getName() + " is not thrown !");
        } finally {
            PetalsJmxApiFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }
}
